<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Memephoto base action controller
 *
 * @author Egor Bulgakov <egor.bulgakov@gmail.com>
 * @package ow.plugin.memephoto.controllers
 * @since 1.0
 */
class MEMEPHOTO_CTRL_Memephoto extends OW_ActionController
{
	/**
	 * @var OW_PluginManager
	 */
	private $plugin;
	/**
	 * @var string
	 */
	private $pluginJsUrl;
	/**
	 * @var string
	 */
	private $ajaxResponder;
	/**
	 * @var MEMEPHOTO_BOL_MemephotoService
	 */
	private $memephotoService;
	/**
	 * @var MEMEPHOTO_BOL_MemephotoAlbumService
	 */
	private $memephotoAlbumService;
	/**
	 * @var BASE_CMP_ContentMenu
	 */
	private $menu;

	/**
	 * Class constructor
	 */
	public function __construct()
	{
		parent::__construct();

		$this->plugin = OW::getPluginManager()->getPlugin('memephoto');
		$this->pluginJsUrl = $this->plugin->getStaticJsUrl();
		$this->ajaxResponder = OW::getRouter()->urlFor('MEMEPHOTO_CTRL_Memephoto', 'ajaxResponder');

		$this->memephotoService = MEMEPHOTO_BOL_MemephotoService::getInstance();

		$this->menu = $this->getMenu();

		if ( !OW::getRequest()->isAjax() )
		{
			OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'memephoto', 'memephoto');
		}
	}

	/**
	 * Returns menu component
	 *
	 * @return BASE_CMP_ContentMenu
	 */
	private function getMenu()
	{
		$validLists = array('featured', 'latest', 'toprated', 'tagged', 'manage');
		$classes = array('ow_ic_push_pin', 'ow_ic_clock', 'ow_ic_star', 'ow_ic_tag', 'ow_ic_picture');

		$checkPrivacy = MEMEPHOTO_BOL_MemephotoService::getInstance()->countMemephotos('featured', true, MEMEPHOTO_BOL_Memephoto::USER_UPLOADED);
		if ( !MEMEPHOTO_BOL_MemephotoService::getInstance()->countMemephotos('featured', $checkPrivacy, MEMEPHOTO_BOL_Memephoto::USER_UPLOADED) )
		{
			array_shift($validLists);
			array_shift($classes);
		}

		$language = OW::getLanguage();

		$menuItems = array();

		$order = 0;
		foreach ( $validLists as $type )
		{
			$item = new BASE_MenuItem();
			$item->setLabel($language->text('memephoto', 'menu_' . $type));
			$item->setUrl(OW::getRouter()->urlForRoute('view_memephoto_list', array('listType' => $type)));
			$item->setKey($type);
			$item->setIconClass($classes[$order]);
			$item->setOrder($order);

			array_push($menuItems, $item);

			$order++;
		}

		$menu = new BASE_CMP_ContentMenu($menuItems);

		return $menu;
	}

	/**
	 * View memephoto action
	 *
	 * @param array $params
	 */
	public function view( array $params )
	{
		if ( !isset($params['id']) || !($memephotoId = (int) $params['id']) )
		{
			throw new Redirect404Exception();
		}

		$memephoto = $this->memephotoService->findMemephotoById($memephotoId);

		if ( !$memephoto )
		{
			throw new Redirect404Exception();
		}

		$language = OW::getLanguage();

		// is owner
		$contentOwner = $memephoto->userId;
		$userId = OW::getUser()->getId();
		$ownerMode = $contentOwner == $userId;
		$this->assign('ownerMode', $ownerMode);
			
		// is moderator
		$modPermissions = OW::getUser()->isAuthorized('memephoto');
		$this->assign('moderatorMode', $modPermissions);

		if ( !$ownerMode && !$modPermissions && !OW::getUser()->isAuthorized('memephoto', 'view') )
		{
			$this->setTemplate(OW::getPluginManager()->getPlugin('base')->getCtrlViewDir() . 'authorization_failed.html');
			return;
		}

		// permissions check
		if ( !$ownerMode && !$modPermissions )
		{
			$privacyParams = array('action' => 'memephoto_view_photo', 'ownerId' => $contentOwner, 'viewerId' => $userId);
			$event = new OW_Event('privacy_check_permission', $privacyParams);
			OW::getEventManager()->trigger($event);
		}

		OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('base')->getStaticJsUrl().'jquery.bbq.min.js');
		OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('memephoto')->getStaticJsUrl() . 'memephoto.js');

		OW::getLanguage()->addKeyForJs('memephoto', 'tb_edit_memephoto');
		OW::getLanguage()->addKeyForJs('memephoto', 'confirm_delete');
		OW::getLanguage()->addKeyForJs('memephoto', 'mark_featured');
		OW::getLanguage()->addKeyForJs('memephoto', 'remove_from_featured');

		$objParams = array(
            'ajaxResponder' => OW::getRouter()->urlFor('MEMEPHOTO_CTRL_Memephoto', 'ajaxResponder'),
            'fbResponder' => OW::getRouter()->urlForRoute('memephoto.floatbox'),
            'layout' => 'page'
            );

            $script =
        'if ( !window.memephotoViewObj ) {
            window.memephotoViewObj = new memephotoView('.json_encode($objParams).');
        }
        
        window.memephotoViewObj.showMemephotoCmp('.$memephoto->id.');
        
        $(window).bind( "hashchange", function(e) {
            var memephoto_id = $.bbq.getState("view-memephoto");
            if ( memephoto_id != undefined )
            {
                if ( window.memephotoFBLoading ) { return; }
                window.memephotoViewObj.showMemephotoCmp(memephoto_id);
            }
        });';

            OW::getDocument()->addOnloadScript($script);
            	
            OW::getDocument()->addStyleDeclaration('.memephoto_description{display: block !important;}');

            //OW::getDocument()->setHeading($memephoto->description);
            //OW::getDocument()->setHeadingIconClass('ow_ic_picture');

            $description = strip_tags($memephoto->description);
            $description = mb_strlen($description) ? $description : $memephoto->id;

            OW::getDocument()->setTitle($language->text('memephoto', 'meta_title_memephoto_view', array('title' => $description)));
            $tagsArr = BOL_TagService::getInstance()->findEntityTags($memephoto->id, 'memephoto');

            foreach ( $tagsArr as $t )
            {
            	$labels[] = $t->label;
            }
            $tagStr = $tagsArr ? implode(', ', $labels) : '';
            OW::getDocument()->setDescription($language->text('memephoto', 'meta_description_memephoto_view', array('title' => $description, 'tags' => $tagStr)));
	}

	/**
	 * Memephoto list action
	 *
	 * @param array $params
	 */
	public function viewList( array $params )
	{
		$listType = isset($params['listType']) ? $params['listType'] : 'latest';

		if($listType == 'manage') {
			$this->redirect(OW::getRouter()->urlForRoute('manage_memephoto'));
		}

		$validLists = array('featured', 'latest', 'toprated', 'tagged');

		if ( !in_array($listType, $validLists) )
		{
			$this->redirect(OW::getRouter()->urlForRoute('view_memephoto_list', array('listType' => 'latest')));
		}
			
		// is moderator
		$modPermissions = OW::getUser()->isAuthorized('memephoto');

		if ( !$modPermissions && !OW::getUser()->isAuthorized('memephoto', 'view') )
		{
			$this->setTemplate(OW::getPluginManager()->getPlugin('base')->getCtrlViewDir() . 'authorization_failed.html');
			return;
		}

		$el = $this->menu->getElement($listType);
		if ( $el )
		{
			$el->setActive(true);
		}

		$this->addComponent('memephotoMenu', $this->menu);
		$this->assign('listType', $listType);

		OW::getDocument()->setHeading(OW::getLanguage()->text('memephoto', 'page_title_browse_memephotos'));
		OW::getDocument()->setHeadingIconClass('ow_ic_picture');

		OW::getDocument()->setTitle(OW::getLanguage()->text('memephoto', 'meta_title_memephoto_'.$listType));
		OW::getDocument()->setDescription(OW::getLanguage()->text('memephoto', 'meta_description_memephoto_'.$listType));

		//OW::getDocument()->addStyleSheet(OW::getPluginManager()->getPlugin('memephoto')->getStaticCssUrl().'bootstrap.css');
		OW::getDocument()->addStyleSheet(OW::getPluginManager()->getPlugin('memephoto')->getStaticCssUrl().'memephoto.css');

		$js = UTIL_JsGenerator::newInstance()
		->newVariable('addNewUrl', OW::getRouter()->urlFor('MEMEPHOTO_CTRL_Memephoto', 'choose'))
		->jQueryEvent('#btn-add-new-memephoto', 'click', 'document.location.href = addNewUrl');

		OW::getDocument()->addOnloadScript($js);

		$script = "
			$('.photoListItem').each(function(){
			    var div = this;
			    var start = $(div).offset().top;
			    var height = $(div).height();   
			    var stickyLeft = $(div).find('.sticky-action').offset().left;
			    var stickyHeight = $(div).find('.sticky-action').height();
			    $.event.add(window, 'scroll', function() {  
			        var p = $(window).scrollTop();
			        var isFixed = ((p)>start) && ((p) < (start + height - stickyHeight));
			        var isFixedAndRelatived = ((p) >= (start + height - stickyHeight) && (p) <= (start + height));
			        $(div).find('.badge-entry-sticky').css('position', isFixed ? 'fixed' : 'relative');
			        $(div).find('.badge-entry-sticky').css('top', isFixed ? '0' : isFixedAndRelatived ? height - stickyHeight + 'px' : '');
			        $(div).find('.badge-entry-sticky').css('z-index', isFixed ? '100' : '1');           
			    });
			})
		";
		OW::getDocument()->addOnloadScript($script);
		$this->getScriptvote();

		$tags = new BASE_CMP_EntityTagCloud('memephoto');
		$tags->setRouteName('view_tagged_memephoto_list');
		$this->addComponent('tags', $tags);
		
		$this->addComponent('topPosters', new MEMEPHOTO_CMP_TopPostersWidget(array('count'=> 10)));
			
	}

	/**
	 * Tagged memephoto list action
	 *
	 * @param array $params
	 */
	public function viewTaggedList( array $params = null )
	{
		if ( isset($params['tag']) )
		{
			$tag = htmlspecialchars(urldecode($params['tag']));
		}

		// is moderator
		$modPermissions = OW::getUser()->isAuthorized('memephoto');

		if ( !$modPermissions && !OW::getUser()->isAuthorized('memephoto', 'view') )
		{
			$this->setTemplate(OW::getPluginManager()->getPlugin('base')->getCtrlViewDir() . 'authorization_failed.html');
			return;
		}

		$this->addComponent('memephotoMenu', $this->menu);

		$this->menu->getElement('tagged')->setActive(true);

		$this->setTemplate(OW::getPluginManager()->getPlugin('memephoto')->getCtrlViewDir() . 'memephoto_view_list-tagged.html');

		$listUrl = OW::getRouter()->urlForRoute('view_tagged_memephoto_list_st');

		OW::getDocument()->addScript($this->pluginJsUrl . 'memephoto_tag_search.js');

		$objParams = array(
            'listUrl' => $listUrl
		);

		$script =
            "$(document).ready(function(){
                var memephotoSearch = new memephotoTagSearch(" . json_encode($objParams) . ");
            }); ";

		OW::getDocument()->addOnloadScript($script);

		if ( isset($tag) )
		{
			$this->assign('tag', $tag);
			OW::getDocument()->setTitle(OW::getLanguage()->text('memephoto', 'meta_title_memephoto_tagged_as', array('tag' => $tag)));
			OW::getDocument()->setDescription(OW::getLanguage()->text('memephoto', 'meta_description_memephoto_tagged_as', array('tag' => $tag)));
		}
		else
		{
			$tags = new BASE_CMP_EntityTagCloud('memephoto');
			$tags->setRouteName('view_tagged_memephoto_list');
			$this->addComponent('tags', $tags);

			OW::getDocument()->setTitle(OW::getLanguage()->text('memephoto', 'meta_title_memephoto_tagged'));
			$tagsArr = BOL_TagService::getInstance()->findMostPopularTags('memephoto', 20);
			foreach ( $tagsArr as $t )
			{
				$labels[] = $t['label'];
			}
			$tagStr = $tagsArr ? implode(', ', $labels) : '';
			OW::getDocument()->setDescription(OW::getLanguage()->text('memephoto', 'meta_description_memephoto_tagged', array('topTags' => $tagStr)));
		}

		$this->assign('listType', 'tagged');

		OW::getDocument()->setHeading(OW::getLanguage()->text('memephoto', 'page_title_browse_memephotos'));
		OW::getDocument()->setHeadingIconClass('ow_ic_picture');

		$js = UTIL_JsGenerator::newInstance()
		->newVariable('addNewUrl', OW::getRouter()->urlFor('MEMEPHOTO_CTRL_Memephoto', 'choose'))
		->jQueryEvent('#btn-add-new-memephoto', 'click', 'document.location.href = addNewUrl');

		OW::getDocument()->addOnloadScript($js);

		OW::getDocument()->addStyleSheet(OW::getPluginManager()->getPlugin('memephoto')->getStaticCssUrl().'memephoto.css');

		$js = UTIL_JsGenerator::newInstance()
		->newVariable('addNewUrl', OW::getRouter()->urlFor('MEMEPHOTO_CTRL_Memephoto', 'choose'))
		->jQueryEvent('#btn-add-new-memephoto', 'click', 'document.location.href = addNewUrl');

		OW::getDocument()->addOnloadScript($js);

		$script = "
			$('.photoListItem').each(function(){
			    var div = this;
			    var start = $(div).offset().top;
			    var height = $(div).height();   
			    var stickyLeft = $(div).find('.sticky-action').offset().left;
			    var stickyHeight = $(div).find('.sticky-action').height();
			    $.event.add(window, 'scroll', function() {  
			        var p = $(window).scrollTop();
			        var isFixed = ((p)>start) && ((p) < (start + height - stickyHeight));
			        var isFixedAndRelatived = ((p) >= (start + height - stickyHeight) && (p) <= (start + height));
			        $(div).find('.badge-entry-sticky').css('position', isFixed ? 'fixed' : 'relative');
			        $(div).find('.badge-entry-sticky').css('top', isFixed ? '0' : isFixedAndRelatived ? height - stickyHeight + 'px' : '');
			        $(div).find('.badge-entry-sticky').css('z-index', isFixed ? '100' : '1');           
			    });
			})
		";
		OW::getDocument()->addOnloadScript($script);

	}

	/**
	 * Controller action for user albums list
	 *
	 * @param unknown_type $params
	 */
	public function userAlbums( array $params )
	{
		if ( empty($params['user']) || !mb_strlen($username = trim($params['user'])) )
		{
			throw new Redirect404Exception();
		}

		$user = BOL_UserService::getInstance()->findByUsername($username);
		if ( !$user )
		{
			throw new Redirect404Exception();
		}

		$userId = $user->id;
		$ownerMode = $userId == OW::getUser()->getId();

		// is moderator
		$modPermissions = OW::getUser()->isAuthorized('memephoto');

		if ( !OW::getUser()->isAuthorized('memephoto', 'view') && !$modPermissions && !$ownerMode )
		{
			$this->setTemplate(OW::getPluginManager()->getPlugin('base')->getCtrlViewDir() . 'authorization_failed.html');
			return;
		}

		// permissions check
		if ( !$ownerMode && !$modPermissions )
		{
			$privacyParams = array('action' => 'memephoto_view_photo', 'ownerId' => $userId, 'viewerId' => OW::getUser()->getId());
			$event = new OW_Event('privacy_check_permission', $privacyParams);
			OW::getEventManager()->trigger($event);
		}

		$this->assign('username', $username);
		$displayName = BOL_UserService::getInstance()->getDisplayName($userId);
		$this->assign('displayName', $displayName);

		$total = $this->memephotoAlbumService->countUserAlbums($userId);
		$page = !empty($_GET['page']) && (int) $_GET['page'] ? abs((int) $_GET['page']) : 1;

		$config = OW::getConfig();
		$albumPerPage = $config->getValue('memephoto', 'memephotos_per_page');

		$albums = $this->memephotoAlbumService->findUserAlbumList($userId, $page, $albumPerPage);
		$this->assign('albums', $albums);
		$this->assign('total', $total);
		$this->assign('userId', $userId);

		// Paging
		$pages = (int) ceil($total / $albumPerPage);
		$paging = new BASE_CMP_Paging($page, $pages, $albumPerPage);
		$this->assign('paging', $paging->render());

		$this->assign('widthConfig', $config->getValue('memephoto', 'preview_image_width'));
		$this->assign('heightConfig', $config->getValue('memephoto', 'preview_image_height'));

		OW::getDocument()->setHeading(
		OW::getLanguage()->text('memephoto', 'page_title_user_albums', array('user' => $displayName))
		);

		OW::getDocument()->setHeadingIconClass('ow_ic_picture');
		OW::getDocument()->setTitle(OW::getLanguage()->text('memephoto', 'meta_title_memephoto_useralbums', array('displayName' => $displayName)));

		if ( $albums )
		{
			$albumTitles = array();
			$i = 0;
			foreach ( $albums as $album )
			{
				$albumTitles[] = $album['dto']->name;
				if ( $i == 10 )
				{
					break;
				}
				$i++;
			}
			$albumTitles = implode(', ', $albumTitles);
			OW::getDocument()->setDescription(
			OW::getLanguage()->text('memephoto', 'meta_description_memephoto_useralbums', array('displayName' => $displayName, 'albums' => $albumTitles))
			);
		}
	}

	/**
	 * Controller action for user album
	 *
	 * @param array $params
	 */
	public function userAlbum( array $params )
	{
		if ( !isset($params['user']) || !strlen($user = trim($params['user'])) )
		{
			throw new Redirect404Exception();
		}

		if ( !isset($params['album']) || !($albumId = (int) $params['album']) )
		{
			throw new Redirect404Exception();
		}

		// is owner
		$userDto = BOL_UserService::getInstance()->findByUsername($user);

		if ( $userDto )
		{
			$ownerMode = $userDto->id == OW::getUser()->getId();
		}
		else
		{
			$ownerMode = false;
		}

		// is moderator
		$modPermissions = OW::getUser()->isAuthorized('memephoto');

		if ( !OW::getUser()->isAuthorized('memephoto', 'view') && !$modPermissions && !$ownerMode )
		{
			$this->setTemplate(OW::getPluginManager()->getPlugin('base')->getCtrlViewDir() . 'authorization_failed.html');
			return;
		}

		$page = !empty($_GET['page']) && (int) $_GET['page'] ? abs((int) $_GET['page']) : 1;

		$config = OW::getConfig();
		$memephotoPerPage = $config->getValue('memephoto', 'memephotos_per_page');

		$album = $this->memephotoAlbumService->findAlbumById($albumId);

		if ( !$album )
		{
			throw new Redirect404Exception();
			return;
		}

		$this->assign('album', $album);

		// permissions check
		if ( !$ownerMode && !$modPermissions )
		{
			$privacyParams = array('action' => 'memephoto_view_photo', 'ownerId' => $album->userId, 'viewerId' => OW::getUser()->getId());
			$event = new OW_Event('privacy_check_permission', $privacyParams);
			OW::getEventManager()->trigger($event);
		}

		$this->assign('userName', BOL_UserService::getInstance()->getUserName($album->userId));
		$displayName = BOL_UserService::getInstance()->getDisplayName($album->userId);
		$this->assign('displayName', $displayName);

		$memephotos = $this->memephotoService->getAlbumMemephotos($albumId, $page, $memephotoPerPage);
		$this->assign('memephotos', $memephotos);

		$total = $this->memephotoAlbumService->countAlbumMemephotos($albumId);
		$this->assign('total', $total);

		$lastUpdated = $this->memephotoAlbumService->getAlbumUpdateTime($albumId);
		$this->assign('lastUpdate', $lastUpdated);

		$this->assign('widthConfig', $config->getValue('memephoto', 'preview_image_width'));
		$this->assign('heightConfig', $config->getValue('memephoto', 'preview_image_height'));

		// Paging
		$pages = (int) ceil($total / $memephotoPerPage);
		$paging = new BASE_CMP_Paging($page, $pages, $memephotoPerPage);
		$this->assign('paging', $paging->render());

		OW::getDocument()->setHeading(
		$album->name .
            ' <span class="ow_small">' .
		OW::getLanguage()->text('memephoto', 'memephotos_in_album', array('total' => $total)) .
            '</span>'
            );

            OW::getDocument()->setHeadingIconClass('ow_ic_picture');

            // check permissions
            $canEdit = OW::getUser()->isAuthorized('memephoto', 'upload', $album->userId);
            $canModerate = OW::getUser()->isAuthorized('memephoto');

            $authorized = $canEdit || $canModerate;
            $this->assign('authorized', $canEdit || $canModerate);
            $this->assign('canUpload', $canEdit);

            $lang = OW::getLanguage();

            if ( $authorized )
            {
            	$albumEditForm = new albumEditForm();
            	$albumEditForm->getElement('albumName')->setValue($album->name);
            	$albumEditForm->getElement('id')->setValue($album->id);

            	$this->addForm($albumEditForm);

            	OW::getDocument()->addScript($this->pluginJsUrl . 'album.js');

            	if ( OW::getRequest()->isPost() && $albumEditForm->isValid($_POST) )
            	{
            		$res = $albumEditForm->process();
            		if ( $res['result'] )
            		{
            			OW::getFeedback()->info($lang->text('memephoto', 'memephoto_album_updated'));
            			$this->redirect();
            		}
            	}

            	$lang->addKeyForJs('memephoto', 'confirm_delete_album');
            	$lang->addKeyForJs('memephoto', 'edit_album');

            	$objParams = array(
                'ajaxResponder' => $this->ajaxResponder,
                'albumId' => $albumId,
                'uploadUrl' => OW::getRouter()->urlForRoute('memephoto_upload_album', array('album' => $album->id))
            	);

            	$script =
                "$(document).ready(function(){
                    var album = new memephotoAlbum( " . json_encode($objParams) . ");
                }); ";

            	OW::getDocument()->addOnloadScript($script);
            }

            OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('base')->getStaticJsUrl().'jquery.bbq.min.js');
            OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('memephoto')->getStaticJsUrl() . 'memephoto.js');

            OW::getLanguage()->addKeyForJs('memephoto', 'tb_edit_memephoto');
            OW::getLanguage()->addKeyForJs('memephoto', 'confirm_delete');
            OW::getLanguage()->addKeyForJs('memephoto', 'mark_featured');
            OW::getLanguage()->addKeyForJs('memephoto', 'remove_from_featured');

            $objParams = array(
            'ajaxResponder' => OW::getRouter()->urlFor('MEMEPHOTO_CTRL_Memephoto', 'ajaxResponder'),
            'fbResponder' => OW::getRouter()->urlForRoute('memephoto.floatbox')
            );

            $script = '$("div.ow_memephoto_list_item_thumb a").on("click", function(e){
            e.preventDefault();
            var memephoto_id = $(this).attr("rel");

            if ( !window.memephotoViewObj ) {
                window.memephotoViewObj = new memephotoView('.json_encode($objParams).');
            }
            
            window.memephotoViewObj.setId(memephoto_id);
        });
        
        $(window).bind( "hashchange", function(e) {
            var memephoto_id = $.bbq.getState("view-memephoto");
            if ( memephoto_id != undefined )
            {
                if ( window.memephotoFBLoading ) { return; }
                window.memephotoViewObj.showMemephotoCmp(memephoto_id);
            }
        });';

            OW::getDocument()->addOnloadScript($script);

            OW::getDocument()->setTitle(
            $lang->text('memephoto', 'meta_title_memephoto_useralbum', array('displayName' => $displayName, 'albumName' => $album->name))
            );
            OW::getDocument()->setDescription(
            $lang->text('memephoto', 'meta_description_memephoto_useralbum', array('displayName' => $displayName, 'number' => $total))
            );
	}

	/**
	 * Method acts as ajax responder. Calls methods using ajax
	 *
	 * @return JSON encoded string
	 *
	 */
	public function ajaxResponder()
	{
		if ( isset($_POST['ajaxFunc']) && OW::getRequest()->isAjax() )
		{
			$callFunc = (string) $_POST['ajaxFunc'];

			$result = call_user_func(array($this, $callFunc), $_POST);
		}
		else
		{
			throw new Redirect404Exception();
			exit;
		}

		exit(json_encode($result));
	}

	/**
	 * Set memephoto approval status (approved | blocked)
	 *
	 * @param array $params
	 * @return array
	 */
	public function ajaxSetApprovalStatus( array $params )
	{
		$memephotoId = $params['memephotoId'];
		$status = $params['status'];

		$isModerator = OW::getUser()->isAuthorized('memephoto');

		if ( !$isModerator )
		{
			throw new Redirect404Exception();
			return;
		}

		$setStatus = $this->memephotoService->updateMemephotoStatus($memephotoId, $status);

		if ( $setStatus )
		{
			$return = array('result' => true, 'msg' => OW::getLanguage()->text('memephoto', 'status_changed'));
		}
		else
		{
			$return = array('result' => false, 'error' => OW::getLanguage()->text('memephoto', 'status_not_changed'));
		}

		return $return;
	}

	public function ajaxUpdateMemephoto()
	{
		if ( OW::getRequest()->isAjax() )
		{
			$memephotoId = (int) $_POST['id'];
			$memephotoService = MEMEPHOTO_BOL_MemephotoService::getInstance();
			$memephoto = $memephotoService->findMemephotoById($memephotoId);
				
			$form = new MEMEPHOTO_CLASS_EditForm($memephoto);
			if ( $form->isValid($_POST) )
			{
				$values = $form->getValues();
				$language = OW::getLanguage();
				if ( $memephoto )
				{

					$memephoto->description = $values['description'];

					BOL_TagService::getInstance()->updateEntityTags(
					$memephoto->id,
                        'memephoto',
					$values['tags']
					);

					if ( $memephotoService->updateMemephoto($memephoto) )
					{
						exit(json_encode(array('result' => true, 'id' => $memephoto->id)));
					}
				}
			}
		}
		//OW::getFeedback()->info($language->text('memephoto', 'memephoto_updated'));
	}

	/**
	 * Set memephoto's 'is featured' status
	 *
	 * @param array $params
	 * @return array
	 */
	public function ajaxSetFeaturedStatus( array $params )
	{
		$memephotoId = $params['memephotoId'];
		$status = $params['status'];

		$isModerator = OW::getUser()->isAuthorized('memephoto');

		if ( !$isModerator )
		{
			throw new Redirect404Exception();
			return;
		}

		$setResult = $this->memephotoService->updateMemephotoFeaturedStatus($memephotoId, $status);

		if ( $setResult )
		{
			$return = array('result' => true, 'msg' => OW::getLanguage()->text('memephoto', 'status_changed'));
		}
		else
		{
			$return = array('result' => false, 'error' => OW::getLanguage()->text('memephoto', 'status_not_changed'));
		}

		return $return;
	}

	/**
	 * Deletes memephoto
	 *
	 * @param array $params
	 * @return array
	 */
	public function ajaxDeleteMemephoto( array $params )
	{
		$memephotoId = $params['memephotoId'];

		$memephoto = $this->memephotoService->findMemephotoById($memephotoId);

		if ( $memephoto )
		{
			$ownerId = $memephoto->userId;
			$isOwner = OW::getUser()->isAuthorized('memephoto', 'upload', $ownerId);
			$isModerator = OW::getUser()->isAuthorized('memephoto');

			if ( !$isOwner && !$isModerator )
			{
				throw new Redirect404Exception();
				return;
			}

			$delResult = $this->memephotoService->deleteMemephoto($memephotoId);

			if ( $delResult )
			{
				$url = OW_Router::getInstance()->urlForRoute('memephoto_list_index');

				$return = array('result' => true, 'msg' => OW::getLanguage()->text('memephoto', 'memephoto_deleted'), 'url' => $url);
			}
			else
			{
				$return = array('result' => false, 'error' => OW::getLanguage()->text('memephoto', 'memephoto_not_deleted'));
			}
		}

		return $return;
	}

	/**
	 * Deletes memephoto album
	 *
	 * @param array $params
	 * @return array
	 */
	public function ajaxDeleteMemephotoAlbum( array $params )
	{
		$albumId = $params['albumId'];
		$lang = OW::getLanguage();

		$album = $this->memephotoAlbumService->findAlbumById($albumId);

		if ( $album )
		{
			// check permissions
			$canEdit = OW::getUser()->isAuthorized('memephoto', 'upload', $album->userId);
			$canModerate = OW::getUser()->isAuthorized('memephoto');

			$authorized = $canEdit || $canModerate;

			if ( $authorized )
			{
				$delResult = $this->memephotoAlbumService->deleteAlbum($albumId);

				if ( $delResult )
				{
					$url = OW_Router::getInstance()->urlForRoute(
                            'memephoto_user_albums',
					array('user' => BOL_UserService::getInstance()->getUserName($album->userId))
					);

					return array('result' => true, 'msg' => $lang->text('memephoto', 'album_deleted'), 'url' => $url);
				}
			}
			else
			{
				$url = OW_Router::getInstance()->urlForRoute(
                        'memephoto_user_album',
				array('user' => BOL_UserService::getInstance()->getUserName($album->userId), 'album' => $album->id)
				);

				return array('result' => false, 'error' => $lang->text('memephoto', 'album_delete_not_allowed'), 'url' => $url);
			}
		}
	}

	public function getFloatbox( )
	{
		if ( empty($_POST['memephotoId']) || !$_POST['memephotoId'] )
		{
			throw new Redirect404Exception();
			exit;
		}

		$memephotoId = (int) $_POST['memephotoId'];

		$service = MEMEPHOTO_BOL_MemephotoService::getInstance();
		$memephoto = $service->findMemephotoById($memephotoId);

		if ( !$memephoto )
		{
			exit(json_encode(array('result' => 'error')));
		}

		// increase view counter
		$memephoto->viewCount++;
		$this->memephotoService->updateMemephoto($memephoto);

		// is moderator
		$moderatorMode = OW::getUser()->isAuthorized('memephoto');
		$userId = OW::getUser()->getId();

		$resp['result'] = "success";

		if ( $_POST['current'] == "true" )
		{
			$resp['current'] = $this->prepareMarkup($memephotoId);

			$contentOwner = $memephoto->userId;
			$ownerMode = $contentOwner == $userId;
			$resp['current']['authorized'] = $ownerMode || $moderatorMode || OW::getUser()->isAuthorized('memephoto', 'view');
		}

		if ( $_POST['prev'] == "true" )
		{
			$prevMemephoto = $service->getPreviousMemephoto($memephoto->id);

			if ( $prevMemephoto )
			{
				$resp['prev'] = $this->prepareMarkup($prevMemephoto['dto']->id);
				$resp['prev']['prev'] = $service->getPreviousMemephotoId($prevMemephoto['dto']->id);
				$resp['prev']['next'] = $memephoto->id;
				$resp['current']['prev'] = $prevMemephoto['dto']->id;

				$contentOwner = $memephoto->userId;
				$ownerMode = $contentOwner == $userId;
				$resp['prev']['authorized'] = $ownerMode || $moderatorMode || OW::getUser()->isAuthorized('memephoto', 'view');
			}
		}

		if ( $_POST['next'] == "true" )
		{
			$nextMemephoto = $service->getNextMemephoto($memephoto->id);

			if ( $nextMemephoto )
			{
				$resp['next'] = $this->prepareMarkup($nextMemephoto['dto']->id);
				$resp['next']['prev'] = $memephoto->id;
				$resp['next']['next'] = $service->getNextMemephotoId($nextMemephoto['dto']->id);
				$resp['current']['next'] = $nextMemephoto['dto']->id;

				$contentOwner = $memephoto->userId;
				$ownerMode = $contentOwner == $userId;
				$resp['next']['authorized'] = $ownerMode || $moderatorMode || OW::getUser()->isAuthorized('memephoto', 'view');
			}
		}


		exit(json_encode($resp));
	}

	private function prepareMarkup( $memephotoId )
	{
		$cmp = new MEMEPHOTO_CMP_MemephotoFloatbox(array('memephotoId' => $memephotoId));

		/* @var $document OW_AjaxDocument */
		$document = OW::getDocument();

		$markup = array();

		$markup['id'] = (int) $memephotoId;
		$markup['html'] = $cmp->render();

		$onloadScript = $document->getOnloadScript();
		if ( !empty($onloadScript) )
		{
			$markup['onloadScript'] = $onloadScript;
		}

		$scriptFiles = $document->getScripts();
		if ( !empty($scriptFiles) )
		{
			$markup['scriptFiles'] = $scriptFiles;
		}

		$css = $document->getStyleDeclarations();
		if ( !empty($css) )
		{
			$markup['css'] = $css;
		}

		return $markup;
	}

	public function choose(){

		// disallow not authenticated access
		if ( !OW::getUser()->isAuthenticated() )
		{
			throw new AuthenticateException();
		}

		// is moderator
		$modPermissions = OW::getUser()->isAuthorized('memephoto');

		if ( !$modPermissions && !OW::getUser()->isAuthorized('memephoto', 'create') )
		{
			$this->setTemplate(OW::getPluginManager()->getPlugin('base')->getCtrlViewDir() . 'authorization_failed.html');
			return;
		}

		$page = !empty($_GET['page']) && (int) $_GET['page'] ? abs((int) $_GET['page']) : 1;
		$memephotosPerPage = 50;
		$memephotos = $this->memephotoService->findMemephotoList('latest', $page, $memephotosPerPage, false, MEMEPHOTO_BOL_Memephoto::ADMIN_UPLOADED);
		$records = $this->memephotoService->countMemephotos('latest', false, MEMEPHOTO_BOL_Memephoto::ADMIN_UPLOADED);

		if ( $memephotos )
		{
			foreach ( $memephotos as $key => $memephoto )
			{
				$memephotos[$key]['url'] = MEMEPHOTO_BOL_MemephotoService::getInstance()->getMemephotoUrl($memephoto['id']);
			}

			// Paging
			$pages = (int) ceil($records / $memephotosPerPage);
			$paging = new BASE_CMP_Paging($page, $pages, 10);
			$this->addComponent('paging', $paging);

			$this->assign('memephotos', $memephotos);
			$this->assign('no_content', false);
		}
		else
		{
			$this->assign('no_content', true);
		}
		OW::getDocument()->addStyleSheet(OW::getPluginManager()->getPlugin('memephoto')->getStaticCssUrl().'bootstrap.css');
	}

	public function create(array $params){

		// disallow not authenticated access
		if ( !OW::getUser()->isAuthenticated() )
		{
			throw new AuthenticateException();
		}

		// is moderator
		$modPermissions = OW::getUser()->isAuthorized('memephoto');

//		if ( !$modPermissions && !OW::getUser()->isAuthorized('memephoto', 'create') )
//		{
//			$this->setTemplate(OW::getPluginManager()->getPlugin('base')->getCtrlViewDir() . 'authorization_failed.html');
//			return;
//		}

		$createMemeForm = new CreateMemeForm();

		$this->addForm($createMemeForm);

		if ( OW::getRequest()->isPost() && $createMemeForm->isValid($_POST) )
		{
			$res = $createMemeForm->process();

			//Newsfeed
			$event = new OW_Event('feed.action', array(
                    'pluginKey' => 'memephoto',
                    'entityType' => 'memephoto_comments',
                    'entityId' => $res->id,
                    'userId' => $res->userId
			));
			OW::getEventManager()->trigger($event);

			$this->redirect(OW::getRouter()->urlForRoute('view_memephoto', array('id' => $res->id)));
		}

		if ( !isset($params['id']) || !($photoId = (int) $params['id']) )
		{
			throw new Redirect404Exception();
		}

		$photo = $this->memephotoService->findMemephotoById($photoId);

		if ( !$photo )
		{
			throw new Redirect404Exception();
		}

		$this->assign('photo', $photo);

		OW::getDocument()->addStyleSheet(OW::getPluginManager()->getPlugin('memephoto')->getStaticCssUrl().'spectrum.css');
		OW::getDocument()->addStyleSheet(OW::getPluginManager()->getPlugin('memephoto')->getStaticCssUrl().'bootstrap.css');

		print '
			<script>
				 var gblImgName = "'.$this->memephotoService->getMemephotoUrl($photo->id).'";
        		 var watermark = "'. OW::getConfig()->getValue('memephoto', 'watermark') .'";
        		 var message_error_form = "'. OW::getLanguage() ->text('base', 'form_validate_common_error_message').'"
			</script>
		';

		OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('memephoto')->getStaticJsUrl() . 'kinetic.min.js');
		OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('memephoto')->getStaticJsUrl() . 'meme.min.js');
		OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('memephoto')->getStaticJsUrl() . 'spectrum.js');

		OW::getDocument()->setHeading(OW::getLanguage()->text('memephoto', 'create_meme'));
		OW::getDocument()->setHeadingIconClass('ow_ic_picture');
	}

	public function manage(){

		// disallow not authenticated access
		if ( !OW::getUser()->isAuthenticated() )
		{
			throw new AuthenticateException();
		}

		if ( !OW::getUser()->isAuthorized('memephoto') )
		{
			$this->setTemplate(OW::getPluginManager()->getPlugin('base')->getCtrlViewDir() . 'authorization_failed.html');
			return;
		}

		$el = $this->menu->getElement('manage');
		if ( $el )
		{
			$el->setActive(true);
		}

		$this->addComponent('memephotoMenu', $this->menu);

		OW::getDocument()->setHeading(OW::getLanguage()->text('memephoto', 'manage'));
		OW::getDocument()->setHeadingIconClass('ow_ic_picture');

		OW::getDocument()->setTitle(OW::getLanguage()->text('memephoto', 'manage'));
		OW::getDocument()->setDescription(OW::getLanguage()->text('memephoto', 'manage'));

		$page = !empty($_GET['page']) && (int) $_GET['page'] ? abs((int) $_GET['page']) : 1;
		$memephotosPerPage = 50;
		$memephotos = $this->memephotoService->findMemephotoList('latest', $page, $memephotosPerPage, false, MEMEPHOTO_BOL_Memephoto::ADMIN_UPLOADED);
		$records = $this->memephotoService->countMemephotos('latest', false, MEMEPHOTO_BOL_Memephoto::ADMIN_UPLOADED);

		if ( $memephotos )
		{
			foreach ( $memephotos as $key => $memephoto )
			{
				$memephotos[$key]['url'] = MEMEPHOTO_BOL_MemephotoService::getInstance()->getMemephotoUrl($memephoto['id']);
			}

			// Paging
			$pages = (int) ceil($records / $memephotosPerPage);
			$paging = new BASE_CMP_Paging($page, $pages, 10);
			$this->addComponent('paging', $paging);

			$this->assign('memephotos', $memephotos);
			$this->assign('no_content', false);
		}
		else
		{
			$this->assign('no_content', true);
		}

		$js = UTIL_JsGenerator::newInstance()
		->newVariable('addNewUrl', OW::getRouter()->urlFor('MEMEPHOTO_CTRL_Upload', 'index'))
		->jQueryEvent('#btn-add-new-memephoto', 'click', 'document.location.href = addNewUrl');
		OW::getDocument()->addOnloadScript($js);

		OW::getDocument()->addStyleSheet(OW::getPluginManager()->getPlugin('memephoto')->getStaticCssUrl().'bootstrap.css');

		$script = '
			$(".btn-memephoto-edit").click(function() {
            var memephoto_id = $(this).attr("rel");
            window.edit_memephoto_floatbox = OW.ajaxFloatBox(
                "MEMEPHOTO_CMP_EditMemephoto",
                { memephotoId : memephoto_id },
                { width : 500, iconClass : "ow_ic_edit", title : "'. OW::getLanguage()->text('memephoto', 'tb_edit_memephoto') .'",
                    onLoad : function() {
                        owForms["memephoto-edit-form"].bind("success", function(data){
                            document.location = "'. OW::getRouter()->urlForRoute('manage_memephoto') .'";
                        });
                    }
                }
            );
        });
        
        $(".memephoto-delete").click(function() {
            var memephoto_id = $(this).attr("rel");

            if ( confirm("'. OW::getLanguage()->text('memephoto', 'confirm_delete') .'") ) {
                $.ajax( {
                    url : "'. OW::getRouter()->urlFor('MEMEPHOTO_CTRL_Memephoto', 'ajaxResponder') .'",
                    type : "POST",
                    data : { ajaxFunc : "ajaxDeleteMemephoto", memephotoId : memephoto_id },
                    dataType : "json",
                    success : function(data) {
                        if ( data.result == true ) {
                            document.location = "'. OW::getRouter()->urlForRoute('manage_memephoto') .'";
                        } else if ( data.error != undefined ) {
                            OW.warning(data.error);
                        }
                    }
                });
            }
            else {
                return false;
            }
        });
		';

		OW::getDocument()->addOnloadScript($script);

	}

	public function rate(){
		$language = OW::getLanguage();
		if ( OW::getRequest()->isAjax() )
		{
			$memephotoId = (int) $_POST['memephotoId'];
			$rating = (int) $_POST['rating'];
			$userId = OW::getUser()->getId();
			$userRating = MEMEPHOTO_BOL_MemephotoRatingService::getInstance()->getUserRatingForMemephoto($memephotoId, $userId);

			$memephoto = MEMEPHOTO_BOL_MemephotoDao::getInstance()->findById($memephotoId);

			if($userRating == null){
				$memephotoRating = new MEMEPHOTO_BOL_Rating();
				$memephotoRating->memephotoId = $memephotoId;
				$memephotoRating->userId = $userId;
				$memephotoRating->rating = $rating;

				//
				$memephoto->rating += $rating;
			}else{
				$memephotoRating = MEMEPHOTO_BOL_MemephotoRatingService::getInstance()->findByMemephotoAndUser($memephotoId, $userId);
					
				//checking hack
				if($memephotoRating->rating == $rating){
					//nothing to do
				}else{
					$memephotoRating->rating = $rating;
					// double: +: reset then adding
					$memephoto->rating += $rating * 2;
				}
			}
			//save
			MEMEPHOTO_BOL_MemephotoDao::getInstance()->save($memephoto);
			MEMEPHOTO_BOL_MemephotoRatingService::getInstance()->save($memephotoRating);

			exit(json_encode(array('result' => true,
             'msg' => $language->text('base', 'rate_cmp_success_message'),
             'obj' => $memephoto,
             'totalVote' => MEMEPHOTO_BOL_MemephotoRatingService::getInstance()->countRatingForMemephoto($memephotoId) .' '. $language->text('qa', 'votes')
			)));

		}

	}

	function getScriptvote(){
		$userId = OW::getUser()->getId();
		$rateLink = OW::getRouter()->urlFor('MEMEPHOTO_CTRL_Memephoto', 'rate');
		$canGuessRate = ($userId)? 1 : 0;
		$rate_cmp_auth_error_message  = OW::getLanguage()->text('base', 'rate_cmp_auth_error_message');
		$rate_cmp_owner_cant_rate_error_message = OW::getLanguage()->text('base', 'rate_cmp_owner_cant_rate_error_message');
		$script = "
			$.each($('.thumbs.canVote'), function(){
	$(this).unbind('click').click(clickfunction = function(){
		var canGuessRate = {$canGuessRate};
		if(!canGuessRate) {OW.error(\"{$rate_cmp_auth_error_message}\"); return;}
        var memephoto_id = $(this).attr('rel');
        var owner_id = $(this).attr('owner-id');
        if(owner_id == {$userId}) {OW.error(\"{$rate_cmp_owner_cant_rate_error_message}\"); return;}
		var rating = ($(this).hasClass('up'))? 1 : -1;
		$.ajax( {
                    url : '{$rateLink}',
                    type : 'POST',
                    data : {memephotoId : memephoto_id,
    				 		rating: rating
    						},
                    dataType : 'json',
                    success : function(data) {
                        if ( data.result == true ) {
                            OW.info(data.msg);
                            
                            var memephoto = data.obj;
                            if(rating == 1){
                            	$('#memephoto-'+ memephoto_id).find('.badge-item-vote-container').first().removeClass('down').addClass('up');
                            }else{
    							$('#memephoto-'+ memephoto_id).find('.badge-item-vote-container').first().removeClass('up').addClass('down');
    						}	
    						
    						$('#memephoto-'+ memephoto_id).find('span.points').first().html(memephoto.rating);
   						
    						$('#memephoto-'+ memephoto_id).find('.thumbs.up').first().removeClass('canVote');
    						$('#memephoto-'+ memephoto_id).find('.thumbs.down').first().removeClass('canVote');
    						if(rating == 1){
    							 $('#memephoto-'+ memephoto_id).find('.thumbs.down').first().addClass('canVote');
    						}else if(rating == -1){
    							 $('#memephoto-'+ memephoto_id).find('.thumbs.up').first().addClass('canVote');
    						}  
    						
    						//bindUI again
    						$('.thumbs').unbind('click');
    						$.each($('.thumbs.canVote'), function(){
								$(this).unbind('click').click(clickfunction)
							});	
                        } else if ( data.error != undefined ) {
                            OW.warning(data.error);
                        }
                    }
                });
    });    
});	
		";
		OW::getDocument()->addOnloadScript($script);
	}

}

/**
 * Album edit form class
 */
class albumEditForm extends Form
{

	/**
	 * Class constructor
	 */
	public function __construct()
	{
		parent::__construct('albumEditForm');

		$language = OW::getLanguage();

		// album id field
		$albumIdField = new HiddenField('id');
		$albumIdField->setRequired(true);
		$this->addElement($albumIdField);

		// album name Field
		$albumNameField = new TextField('albumName');

		$this->addElement($albumNameField->setLabel($language->text('memephoto', 'album')));

		$submit = new Submit('save');
		$submit->setValue($language->text('memephoto', 'btn_edit'));
		$this->addElement($submit);
	}

	/**
	 * Updates memephoto album
	 *
	 * @return boolean
	 */
	public function process()
	{
		$values = $this->getValues();

		$albumService = MEMEPHOTO_BOL_MemephotoAlbumService::getInstance();

		$language = OW::getLanguage();

		if ( isset($values['id']) && ($albumId = (int) $values['id']) )
		{
			$album = $albumService->findAlbumById($albumId);

			if ( $album )
			{
				if ( strlen($albumName = htmlspecialchars(trim($values['albumName']))) )
				{
					$album->name = $albumName;

					if ( $albumService->updateAlbum($album) )
					{
						return array('result' => true, 'id' => $album->id);
					}
				}
			}
		}
		else
		{
			return array('result' => false, 'id' => $album->id);
		}

		return false;
	}

}

class CreateMemeForm extends Form
{

	/**
	 * Class constructor
	 *
	 */
	public function __construct()
	{
		parent::__construct('createMemeForm');
		$this->setId('createimg');
		$this->setEnctype(Form::ENCTYPE_MULTYPART_FORMDATA);

		$language = OW::getLanguage();

		$description = new TextField('description');
		$description->setLabel(OW::getLanguage()->text('memephoto', 'title'));
		$description->setRequired(true);
		$description->setId('description');
		$this->addElement($description);

		$tagsField = new TagsInputField('tags');
		$this->addElement($tagsField->setLabel($language->text('memephoto', 'tags')));

		// submit
		$submit = new Button('save');
		$submit->setId('cands');
		$submit->setValue(OW::getLanguage()->text('memephoto', 'create'));
		$this->addElement($submit);

	}

	/**
	 * Updates photo plugin configuration
	 *
	 * @return boolean
	 */
	public function process()
	{
		$memephotoService = MEMEPHOTO_BOL_MemephotoService::getInstance();
		$values = $_POST;
		$imgdata = substr($values['imgdata'], strpos($values['imgdata'], ",") + 1);

		$decodedImage = base64_decode($imgdata);
		$main = MEMEPHOTO_BOL_MemephotoTemporaryDao::getInstance()->getTemporaryMemephotoPath(11, 2);
		//var_dump($decodedImage);

		$description = $values['description'];

		//save
		$privacy = OW::getEventManager()->call(
            'plugin.privacy.get_privacy', 
		array('ownerId' => OW::getUser()->getId(), 'action' => 'memephoto_view_photo')
		);

		$memephoto = new MEMEPHOTO_BOL_Memephoto();
		$memephoto->userId = OW::getUser()->getId();
		$memephoto->description = htmlspecialchars($description);
		$memephoto->addDatetime = time();
		$memephoto->status = 'approved';
		$memephoto->hasFullsize = 0;
		$memephoto->privacy = mb_strlen($privacy) ? $privacy : 'everybody';
		$memephoto->isSystem = MEMEPHOTO_BOL_Memephoto::USER_UPLOADED;
		$memephoto->viewCount = 0;
		$memephoto->rating = 0;

		if ($id = $memephotoService->addMemephoto($memephoto) )
		{
			$preview = $memephotoService->getMemephotoPath($id, 'preview');
			$main = $memephotoService->getMemephotoPath($id);
			$original = $memephotoService->getMemephotoPath($id, 'original');

			$config = OW::getConfig();
			$width = $config->getValue('memephoto', 'main_image_width');
			$height = $config->getValue('memephoto', 'main_image_height');
			$previewWidth = $config->getValue('memephoto', 'preview_image_width');
			$previewHeight = $config->getValue('memephoto', 'preview_image_height');

			try {
				$image = new UTIL_Image($decodedImage);

				$mainMemephoto = $image
				->resizeImage($width, $height)
				->saveImage($main);

				$mainMemephoto
				->resizeImage($previewWidth, $previewHeight, true)
				->saveImage($preview);
			}
			catch ( WideImage_Exception $e )
			{
				MEMEPHOTO_BOL_MemephotoDao::getInstance()->deleteById($id);
			}

			// add tags
			BOL_TagService::getInstance()->updateEntityTags(
			$memephoto->id,
                        'memephoto',
			explode(",", $values['tags'])
			);

		}

		return $memephoto;
	}
}